import time

from ecsclient import schemas
from tests import functional


class TestVDC(functional.BaseTestCase):
    def setUp(self):
        super(TestVDC, self).setUp()
        r = self.client.vdc.get_local()
        self.vdc_1_id = r['id']
        self.vdc_1_name = r['name']
        self.vdc_1_endpoint = r['interVdcEndPoints']

    def test_vdc_list(self):
        response = self.client.vdc.list()
        self.assertValidSchema(response, schemas.VDCS)

    def test_vdc_get_by_id(self):
        response = self.client.vdc.get(vdc_id=self.vdc_1_id)
        self.assertValidSchema(response, schemas.VDC)
        self.assertEqual(response['id'], self.vdc_1_id)
        self.assertEqual(response['vdcId'], self.vdc_1_id)

    def test_vdc_get_by_name(self):
        response = self.client.vdc.get(name=self.vdc_1_name)
        self.assertValidSchema(response, schemas.VDC)
        self.assertEqual(response['name'], self.vdc_1_name)
        self.assertEqual(response['vdcName'], self.vdc_1_name)

    def test_vdc_get_local(self):
        response = self.client.vdc.get_local()
        self.assertValidSchema(response, schemas.VDC)
        self.assertTrue(response['local'])

    def test_vdc_get_local_secret_key(self):
        response = self.client.vdc.get_local_secret_key()
        self.assertIn('key', response)
        self.assertIsNotNone(response['key'])

    def test_vdc_update(self):
        vdc_name = 'vdc-%d' % time.time()
        secret_key = '%d' % time.time()
        self.client.vdc.update(self.vdc_1_name,
                               new_name=vdc_name,
                               secret_key=secret_key,
                               inter_vdc_endpoints=self.vdc_1_endpoint,
                               inter_vdc_cmd_endpoints=self.vdc_1_endpoint,
                               management_endpoints=self.vdc_1_endpoint
                               )

        r = self.client.vdc.get(vdc_id=self.vdc_1_id)
        self.assertEqual(r['name'], vdc_name)
        self.assertEqual(r['vdcName'], vdc_name)
        self.assertEqual(r['secretKeys'], secret_key)

    def test_vdc_delete(self):
        self.skipTest('Cannot create a scenario to delete a VDC')



import logging

from ecsclient.common.provisioning.node import Node

log = logging.getLogger(__name__)


class Node(Node):

    def get_vdc_lock_status(self):
        raise NotImplementedError()

    def set_vdc_lock_status(self):
        raise NotImplementedError()

    def get_node_lock_status(self):
        raise NotImplementedError()

    def set_node_lock_status(self):
        raise NotImplementedError()



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestPasswordGroup(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestPasswordGroup, self).__init__(*args, **kwargs)
        self.namespace_1 = "functional-tests-namespace-%s" % int(time.time())
        self.object_user_1 = "functional-tests-objectuser-%s" % int(time.time())
        self.object_user_2 = self.object_user_1 + '_second'
        self.password_1 = "fake-password-123"
        self.password_2 = "fake-password-456"
        self.groups_list_1 = ['admin']
        self.groups_list_2 = ['admin', 'user']

    def setUp(self):
        super(TestPasswordGroup, self).setUp()
        self.client.namespace.create(self.namespace_1)
        self.client.object_user.create(self.object_user_1, self.namespace_1)
        self.client.password_group.create(self.object_user_1,
                                          self.password_1,
                                          self.groups_list_1,
                                          namespace=self.namespace_1)

    def tearDown(self):
        super(TestPasswordGroup, self).tearDown()
        for object_user in [self.object_user_1,
                            self.object_user_2]:
            try:
                self.client.object_user.delete(object_user)
            except ECSClientException:
                pass
        self.client.namespace.delete(self.namespace_1)

    def test_password_group_create_for_user(self):
        self.client.password_group.create(user_id=self.object_user_2,
                                          namespace=self.namespace_1,
                                          password=self.password_2,
                                          groups_list=self.groups_list_2)

        response = self.client.password_group.get(user_id=self.object_user_2,
                                                  namespace=self.namespace_1)

        self.assertValidSchema(response, schemas.GROUP_LIST)
        self.assertEqual(response['groups_list'], self.groups_list_2)

    def test_password_group_get_by_user(self):
        response = self.client.password_group.get(user_id=self.object_user_1,
                                                  namespace=self.namespace_1)

        self.assertValidSchema(response, schemas.GROUP_LIST)
        self.assertEqual(response['groups_list'], self.groups_list_1)

    def test_password_group_update_for_user(self):
        self.assertNotEqual(self.groups_list_1, self.groups_list_2)

        response = self.client.password_group.get(user_id=self.object_user_1,
                                                  namespace=self.namespace_1)
        self.assertEqual(response['groups_list'], self.groups_list_1)

        self.client.password_group.update(user_id=self.object_user_1,
                                          namespace=self.namespace_1,
                                          password=self.password_2,
                                          groups_list=self.groups_list_2)

        response = self.client.password_group.get(user_id=self.object_user_1,
                                                  namespace=self.namespace_1)
        self.assertEqual(response['groups_list'], self.groups_list_2)

    def test_password_group_delete_for_user(self):
        self.client.password_group.delete(user_id=self.object_user_1,
                                          namespace=self.namespace_1)
        f = self.client.password_group.get
        self.assertRaises(ECSClientException, f, self.object_user_1)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Dashboard(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_local_zone(self, *args, **kwargs):
        """
        Gets the local VDC details

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting local VDC info")
        return self.conn.get(url='dashboard/zones/localzone', params=param)

    def get_local_zone_replication_groups(self, *args, **kwargs):
        """
        Gets the local VDC replication groups details

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting vpools in local VDC")
        return self.conn.get(url='dashboard/zones/localzone/replicationgroups', params=param)

    def get_local_zone_rglinks_failed(self, *args, **kwargs):
        """
        Gets the local VDC replication group failed links details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'_embedded': {
                u'_instances': [

                ]
            },
            u'_links': {
                u'self': {
                    u'href': u'/dashboard/zones/localzone/rglinksFailed'
                }
            },
            u'title': u'rglinksFailedList'
        }
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting failed links for vpools in local VDC")
        return self.conn.get(url='dashboard/zones/localzone/rglinksFailed')

    def get_local_zone_storage_pools(self, *args, **kwargs):
        """
        Gets the local VDC storage pool details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting varrays in local VDC")
        return self.conn.get(url='dashboard/zones/localzone/storagepools', params=param)

    def get_local_zone_nodes(self, *args, **kwargs):
        """
        Gets the local vdc node details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting nodes in local VDC")
        return self.conn.get(url='dashboard/zones/localzone/nodes', params=param)

    def get_storage_pool(self, storage_pool_id, *args, **kwargs):
        """
        Gets the storage pool details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param storage_pool_id: Storage pool identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for varray '{0}'".format(storage_pool_id))

        return self.conn.get(
            url='dashboard/storagepools/{0}'.format(storage_pool_id), params=param)

    def get_node(self, node_id, *args, **kwargs):
        """
        Gets the node instance details

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param node_id: Node identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for node '{0}'".format(node_id))

        return self.conn.get(
            url='dashboard/nodes/{0}'.format(node_id))

    def get_disk(self, disk_id, *args, **kwargs):
        """
        Gets the disk instance details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param disk_id: Storage pool identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for disk '{0}'".format(disk_id))

        return self.conn.get(
            url='dashboard/disks/{0}'.format(disk_id), params=param)

    def get_process(self, process_id, *args, **kwargs):
        """
        Gets the process instance details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param process_id: Identity of the process
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for PID '{0}'".format(process_id))

        return self.conn.get(
            url='dashboard/processes/{0}'.format(process_id), params=param)

    def get_node_processes(self, node_id, *args, **kwargs):
        """
        Gets the node instance process details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param node_id: Identity of the process
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting processes for node '{0}'".format(node_id))

        return self.conn.get(
            url='dashboard/nodes/{0}/processes'.format(node_id), params=param)

    def get_node_disks(self, node_id, *args, **kwargs):
        """
        Gets the node instance disk details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param node_id: Identity of the node
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting disks for node '{0}'".format(node_id))

        return self.conn.get(
            url='dashboard/nodes/{0}/disks'.format(node_id), params=param)

    def get_storage_pool_nodes(self, storage_pool_id, *args, **kwargs):
        """
        Gets the storage pool node details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param storage_pool_id: Identity of the storage pool
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category', 'cfTimeFrame', 'cfTarget']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting nodes for varray '{0}'".format(storage_pool_id))

        return self.conn.get(
            url='dashboard/storagepools/{0}/nodes'.format(storage_pool_id), params=param)

    def get_local_zone_replication_group_bootstrap_links(self, *args, **kwargs):
        """
        Gets the local VDC replication group bootstrap links details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'_embedded': {
                u'_instances': [

                ]
            },
            u'_links': {
                u'self': {
                    u'href': u'/dashboard/zones/localzone/rglinksBootstrap'
                }
            },
            u'title': u'rglinksBootstrapList'
        }
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting vpool bootstrap links in local VDC")

        return self.conn.get(
            url='dashboard/zones/localzone/rglinksBootstrap', params=param)

    def get_replication_group(self, replication_group_id, *args, **kwargs):
        """
        Gets the replication group instance details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param replication_group_id: Replication group identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for vpool '{0}'".format(replication_group_id))

        return self.conn.get(
            url='dashboard/replicationgroups/{0}'.format(replication_group_id), params=param)

    def get_replication_group_link(self, rglink_id, *args, **kwargs):
        """
        Gets the replication group link instance details

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param rglink_id: Replication group link identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting info for vpool link '{0}'".format(rglink_id))
        return self.conn.get(url='dashboard/rglinks/{0}'.format(rglink_id), params=param)

    def get_replication_group_links(self, replication_group_id, *args, **kwargs):
        """
        Gets the replication group instance associated link details.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        Too large to output here

        :param replication_group_id: Replication group identifier
        """
        keys = ['dataType', 'startTime', 'endTime', 'interval', 'category']

        param = {}
        for key in keys:
            value = kwargs.get(key)
            if value:
                param[key] = value
        log.info("Getting links for vpool '{0}'".format(replication_group_id))

        return self.conn.get(
            url='dashboard/replicationgroups/{0}/rglinks'.format(
                replication_group_id),
            params=param
        )



import logging

log = logging.getLogger(__name__)


class Certificate(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_certificate_chain(self):
        """
        Gets the certificate chain currently used. The data returned will
        contain all of the certificates in the current certificate chain
        encoded in PEM format.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'chain': u'-----BEGINCERTIFICATE-----\nMIIEnTCCA4WgAwI-
            REALLY-LONG-CERT\r\n-----ENDCERTIFICATE-----'
        }
        """
        log.info("Fetching certificate chain")
        return self.conn.get('object-cert/keystore')

    def set_certificate_chain(self, selfsigned=False, ip_addresses=None, private_key=None, certificate_chain=None):
        """
        Sets private key and certificate pair. The new certificate and key
        will be rotated into all of the nodes within 1 hour.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API

        {
            'chain': '-----BEGINCERTIFICATE-----\nMIIEnTCCA4WgAwI-
            REALLY-LONG-CERT\r\n-----ENDCERTIFICATE-----'
        }

        :param selfsigned: Set true if the you want the system to generate a new self-signed certificate,
        false otherwise
        :param ip_addresses: List of IP addresses. The IP addresses are taken into account only if selfsigned
        is set to true. i.e, User wants the system to generate a new self-signed certificate
        :param private_key: The private key used to sign the certificate in PEM format
        :param certificate_chain: New certificate for the nodes in PEM format. For certificates signed by an
        intermediate CA (most are), the intermediate certificate(s) should be concatenated to the text string,
        also in PEM format.
        """
        payload = {
            "system_selfsigned": selfsigned
        }
        if selfsigned:
            payload["ip_addresses"] = ip_addresses
        else:
            payload["key_and_certificate"] = {
                "private_key": private_key,
                "certificate_chain": certificate_chain
            }
        log.info("Setting certificate chain")
        return self.conn.put('object-cert/keystore', json_payload=payload)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Events(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_audit_events(self, start_time, end_time, namespace, limit=10,
                         marker=None):
        """
        Gets audit events for the specified namespace identifier and interval.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'MaxEvents': 2,
            u'NextMarker': u'kNDVkLTRkYzYtOWUxNy03MGFkYzAzMWUxNDQ=',
            u'auditevent': [
                {
                    u'serviceType': u'Bucket',
                    u'description': u'BucketCreated',
                    u'eventType': u'Created',
                    u'userId': u'user1',
                    u'timestamp': u'2015-06-08T01: 30',
                    u'namespace': u'namespace1',
                    u'resourceId': u'namespace1.ecstest-2a925333-5f91-43d0-b1',
                    u'id': u'urn: storageos: Event: 29ffd597-7c64-41ef-bc5c-e2'
                },
                {
                    u'serviceType': u'Bucket',
                    u'description': u'BucketCreated',
                    u'eventType': u'Created',
                    u'userId': u'someone1',
                    u'timestamp': u'2015-06-08T01: 49',
                    u'namespace': u'namespace1',
                    u'resourceId': u'namespace1.ecstest-18880408-64ed-4380-80',
                    u'id': u'urn: storageos: Event: 1ee5c8c2-be67-478a-86ac'
                }
            ]
        }

        Time format is: yyyy-MM-dd'T'HH:mm
        Example: 2015-01-25T04:05

        :param start_time: Start time for the interval to retrieve audit events
        :param end_time: End time for the interval to retrieve audit events
        :param namespace: Namespace identifier for which audit events needs to
        be retrieved.
        :param limit: Number of audit events requested in current fetch.
        :param marker: Reference of last audit event returned
        """
        log.info("Getting audit events for namespace '{0}'".format(namespace))

        params = {
            'start_time': start_time,
            'end_time': end_time,
            'namespace': namespace,
            'limit': limit
        }

        if marker:
            params['marker'] = marker

        return self.conn.get(url='vdc/events', params=params)



import logging


log = logging.getLogger(__name__)


class ObjectUser(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self, namespace=None):
        """
        Gets identifiers for all configured users. If namespace is provided
        then returns all users for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blobuser': [
                {
                    u'userid': u'johndoe',
                    u'namespace': u'namespace1'
                },
                {
                    u'userid': u'janedoe',
                    u'namespace': u'namespace1'
                }
            ]
        }

        :param namespace: Namespace for which users should be returned. Optional.
        """
        msg = 'Listing all object users'
        url = 'object/users'

        if namespace:
            url += '/{}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def get(self, user_id, namespace=None):
        """
        Gets user details for the specified user.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'locked': False,
            u'namespace': u'namespace1',
            u'name': u'someone',
            u'created': u'ThuMay2105: 43: 27UTC2015'
        }

        :param user_id: Valid user identifier
        :param namespace: Optional when user scope is GLOBAL. Required when
        user scope is NAMESPACE. The namespace to which the user belongs
        """
        msg = "Getting user ID '{}'".format(user_id)
        url = 'object/users/{}/info'.format(user_id)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url=url)

    def delete(self, user_id, namespace=None):
        """
        Deletes the specified user and its secret keys.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: Valid user identifier
        :param namespace: Example: namespace1 (optional)
        """
        payload = {"user": user_id}
        if namespace:
            payload["namespace"] = namespace

        log.info("Deleting user ID '{}'".format(user_id))
        return self.conn.post('object/users/deactivate', json_payload=payload)

    def create(self, user_id, namespace, tags=None):
        """
        Creates a user for a specified namespace. The user must subsequently
        be assigned a secret key in order to access the object store.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "link": {
              "href": "/object/user-secret-keys/wuser1@sanity.local",
              "rel": "self"
          }
        }

        :param user_id: User to be created
        :param namespace: Namespace identifier to associate with the user
        :param tags: A list of arbitrary tags to assign to the new user. These can
        be used to track additional information about the user and will also
        appear on bucket billing responses for buckets owned by the user
        """

        payload = {
            "user": user_id,
            "namespace": namespace
        }

        if tags:
            payload['tags'] = tags

        log.info('Creating user: {0}'.format(payload))
        return self.conn.post('object/users', json_payload=payload)

    def lock(self, user_id, namespace=None):
        """
        Locks the specified user. If the user belongs to a
        namespace, the namespace must be supplied.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: User name to be locked
        :param namespace: Namespace for this user (optional)
        """
        return self.__lock(user_id, namespace, True)

    def unlock(self, user_id, namespace=None):
        """
        Unlocks the specified user. If the user belongs to a
        namespace, the namespace must be supplied.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: User name to be unlocked
        :param namespace: Namespace for this user (optional)
        """
        return self.__lock(user_id, namespace, False)

    def __lock(self, user_id, namespace, lock):
        payload = {
            "user": user_id,
            "isLocked": lock
        }

        if namespace:
            payload["namespace"] = namespace

        verb = "Locking" if lock else "Unlocking"
        log.info("{} user ID {}".format(verb, user_id))
        return self.conn.put(url='object/users/lock', json_payload=payload)

    def get_lock(self, user_id, namespace=None):
        """
        Gets the user lock state for the specified user belonging to the
        specified namespace (if provided).

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'user_name': u'testlogin',
            u'isLocked': False
        }

        :param user_id: User ID for which user lock status should be returned
        :param namespace: Namespace to which user belongs (optional)
        """
        msg = "Getting lock state for user ID '{}'".format(user_id)
        url = 'object/users/lock/{}'.format(user_id)
        if namespace:
            msg += " in namespace '{}'".format(namespace)
            url += '/{}'.format(namespace)

        log.info(msg)
        return self.conn.get(url)



import time

from ecsclient import schemas
from tests import functional


class TestDataStore(functional.BaseTestCase):
    def setUp(self):
        super(TestDataStore, self).setUp()

        # Get a data store ID
        r = self.client.data_store.list()
        self.data_store_1_id = r['data_store'][0]['id']

        # Get a storage pool ID
        r = self.client.data_store.get(self.data_store_1_id)
        self.storage_pool_1_id = r['varray']

        # Create a new data store to get a task ID
        self.data_store_2_name = "functional-tests-datastore-%s" % int(time.time())
        r = self.client.data_store.create(name=self.data_store_2_name,
                                          description="Data Store for functional test",
                                          node_id=self.data_store_1_id,
                                          storage_pool_id=self.storage_pool_1_id)
        self.task_1_id = r['task'][0]['op_id']

    def test_data_store_list(self):
        response = self.client.data_store.list()
        self.assertValidSchema(response, schemas.DATA_STORES)

    def test_data_store_get(self):
        response = self.client.data_store.get(self.data_store_1_id)
        self.assertValidSchema(response, schemas.DATA_STORE)
        self.assertEqual(response['varray'], self.storage_pool_1_id)

    def test_data_store_get_by_storage_pool(self):
        response = self.client.data_store.get_by_storage_pool(self.storage_pool_1_id)
        self.assertValidSchema(response, schemas.DATA_STORES_COMMODITY)
        self.assertEqual(response['commodity_data_store'][0]['varray'], self.storage_pool_1_id)

    def test_data_store_create(self):
        response = self.client.data_store.create(name=self.data_store_2_name,
                                                 description="Data Store for functional test",
                                                 node_id=self.data_store_1_id,
                                                 storage_pool_id=self.storage_pool_1_id)
        self.assertValidSchema(response, schemas.DATA_STORE_TASKS)

    def test_data_store_delete(self):
        # TODO: API is returning an error. Need to investigate
        self.skipTest('Skipping until investigated')
        # Try to delete a non-existent data store
        response = self.client.data_store.delete(data_store_id='9.9.9.9')
        self.assertValidSchema(response, schemas.DATA_STORE_TASKS)

    def test_data_store_get_task(self):
        response = self.client.data_store.get_task(data_store_id=self.data_store_1_id, op_id=self.task_1_id)
        self.assertValidSchema(response, schemas.DATA_STORE_TASK)
        self.assertEqual(response['op_id'], self.task_1_id)



import testtools
from requests_mock.contrib import fixture
from mock import MagicMock
from mock import mock
from six.moves import http_client
from ecsclient.client import Client
from ecsclient.common.exceptions import ECSClientException


class TestNode(testtools.TestCase):

    def setUp(self):
        super(TestNode, self).setUp()
        self.ecs_endpoint = 'https://127.0.0.1:4443'
        self.token_endpoint = 'https://127.0.0.1:4443/login'

        self.client = Client(
            '2',
            ecs_endpoint=self.ecs_endpoint,
            token_endpoint=self.token_endpoint,
            username='user',
            password='password'
        )

        self.returned_json = {
            "node": [
                {
                    "ip": "172.29.3.148",
                    "version": "1.2.0.0.60071.ffbe16c",
                    "rackId": "gray",
                    "nodename": "supr01-r01-01.lax01s1.rspaas-lab.ops.com",
                    "nodeid": "171.29.3.140"
                },
                {
                    "ip": "172.29.3.149",
                    "version": "1.2.0.0.60071.ffbe16c",
                    "rackId": "gray",
                    "nodename": "supr01-r01-02.lax01s1.rspaas-lab.ops.com",
                    "nodeid": "171.29.3.141"
                }
            ]
        }

        self.response = MagicMock()
        self.requests_mock = self.useFixture(fixture.Fixture())

    @mock.patch('ecsclient.common.token_request.TokenRequest.get_token')
    def test_list_nodes_throw_exception(self, mock_get_token):
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/vdc/nodes',
                                        status_code=http_client.INTERNAL_SERVER_ERROR,
                                        text='Server Error')
        mock_get_token.return_value = 'FAKE-TOKEN-123'

        with super(testtools.TestCase, self).assertRaises(ECSClientException) as error:
            self.client.node.list()

        exception = error.exception
        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, 'https://127.0.0.1:4443/vdc/nodes')
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'FAKE-TOKEN-123')
        self.assertEqual(exception.http_response_content, 'Server Error')
        self.assertEqual(exception.http_status, http_client.INTERNAL_SERVER_ERROR)

    @mock.patch('ecsclient.common.token_request.TokenRequest.get_token')
    def test_list_nodes(self, mock_get_token):
        mock_get_token.return_value = 'FAKE-TOKEN-123'
        self.requests_mock.register_uri('GET', 'https://127.0.0.1:4443/vdc/nodes',
                                        status_code=http_client.OK,
                                        json=self.returned_json)

        response = self.client.node.list()

        self.assertEqual(self.requests_mock.last_request.method, 'GET')
        self.assertEqual(self.requests_mock.last_request.url, 'https://127.0.0.1:4443/vdc/nodes')
        self.assertEqual(self.requests_mock.last_request.headers['x-sds-auth-token'], 'FAKE-TOKEN-123')
        self.assertEqual(response, self.returned_json)



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestManagementUser(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestManagementUser, self).__init__(*args, **kwargs)
        self.management_user_1 = "functional-tests-managementuser-%s" % int(time.time())
        self.management_user_2 = self.management_user_1 + '_second'

    def setUp(self):
        super(TestManagementUser, self).setUp()
        self.client.management_user.create(self.management_user_1,
                                           password='fake-password-123')

    def tearDown(self):
        super(TestManagementUser, self).tearDown()
        for management_user in [self.management_user_1,
                                self.management_user_2]:
            try:
                self.client.management_user.delete(management_user)
            except ECSClientException:
                pass

    def test_management_user_list(self):
        response = self.client.management_user.list()
        self.assertValidSchema(response, schemas.MANAGEMENT_USERS)

    def test_management_user_get(self):
        response = self.client.management_user.get(self.management_user_1)
        self.assertValidSchema(response, schemas.MANAGEMENT_USER)
        self.assertEqual(response['userId'], self.management_user_1)

    def test_management_user_create(self):
        response = self.client.management_user.create(self.management_user_2,
                                                      password='fake-password-123',
                                                      is_system_admin=True,
                                                      is_system_monitor=True,
                                                      is_security_admin=True)
        self.assertValidSchema(response, schemas.MANAGEMENT_USER)
        self.assertEqual(response['userId'], self.management_user_2)
        self.assertTrue(response['isSystemAdmin'])
        self.assertTrue(response['isSystemMonitor'])
        self.assertTrue(response['isSecurityAdmin'])

    def test_management_user_delete(self):
        self.client.management_user.delete(self.management_user_1)
        f = self.client.management_user.get
        self.assertRaises(ECSClientException, f, self.management_user_1)

    def test_management_user_update(self):
        response = self.client.management_user.get(self.management_user_1)
        self.assertFalse(response['isSystemAdmin'])
        self.assertFalse(response['isSystemMonitor'])
        self.assertFalse(response['isSecurityAdmin'])

        self.client.management_user.update(self.management_user_1,
                                           password='fake-password-123',
                                           is_system_admin=True,
                                           is_system_monitor=True,
                                           is_security_admin=True)

        response = self.client.management_user.get(self.management_user_1)
        self.assertTrue(response['isSystemAdmin'])
        self.assertTrue(response['isSystemMonitor'])
        self.assertTrue(response['isSecurityAdmin'])



from ecsclient import schemas
from tests import functional


class TestNode(functional.BaseTestCase):

    def test_node_list(self):
        response = self.client.node.list()
        self.assertValidSchema(response, schemas.NODE_LIST)



import logging

log = logging.getLogger(__name__)


class Namespace(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Gets the identifiers for all configured namespaces.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'namespace': [
                {
                    u'link': {
                        u'href': u'/object/namespaces/namespace/namespace1',
                        u'rel': u'self'
                    },
                    u'name': u'namespace1',
                    u'id': u'namespace1'
                }
            ]
        }
        """
        log.info("Getting all namespaces")
        return self.conn.get(url='object/namespaces')

    def get(self, namespace):
        """
        Gets the details for the given namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'default_data_services_vpool':
                u'urn: storageos: ReplicationGroupInfo:
                    4ea1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global',
            u'remote': None,
            u'name': u'namespace1',
            u'tags': [

            ],
            u'namespace_admins': u'someone@email.com',
            u'user_mapping': [

            ],
            u'global': None,
            u'disallowed_vpools_list': [

            ],
            u'vdc': None,
            u'inactive': False,
            u'link': {
                u'href': u'/object/namespaces/namespace/namespace1',
                u'rel': u'self'
            },
            u'allowed_vpools_list': [

            ],
            u'id': u'namespace1'
        }

        :param namespace: Namespace identifier for which details needs to
        be retrieved.
        """
        log.info("Getting info for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}'.format(namespace))

    def create(self, name, default_object_project=None, default_data_services_vpool=None,
               default_bucket_block_size=None, allowed_vpools_list=[], disallowed_vpools_list=[],
               namespace_admins=None, external_group_admins=None, user_mapping=[],
               is_encryption_enabled=False, is_stale_allowed=False, is_compliance_enabled=False):
        """
        Creates a namespace with the given details

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
          "namespace": {
            "id": "jt_namespace2",
            "inactive": "false",
            "isStaleAllowed": "true",
            "link": {
              "-rel": "self",
              "-href": "/object/namespaces/namespace/jt_namespace2"
            },
            "name": "jt_namespace2",
            "namespace_admins": "admin1,admin2,admin3",
            "user_mapping": [
              {
                "attributes": {
                  "attribute": [
                    {
                      "key": "key1",
                      "value": "value1"
                    },
                    {
                      "key": "key2",
                      "value": "value2"
                    }
                  ]
                },
                "domain": "jt_domain",
                "groups": { "group": "group1" }
              },
              {
                "attributes": {
                  "attribute": [
                    {
                      "key": "key2_1",
                      "value": "value2_1"
                    },
                    {
                      "key": "key2_2",
                      "value": "value2_2"
                    }
                  ]
                },
                "domain": "jt_domain_2",
                "groups": { "group": "group2" }
              }
            ]
          }
        }

        :param name: User provided namespace (verified unique). Cannot include dots or slashes (.|/) in the name
        :param default_object_project: Default project id for this tenant when creating buckets
        :param default_data_services_vpool: Default replication group identifier for this tenant when creating buckets
        :param default_bucket_block_size: Default bucket quota size
        :param allowed_vpools_list: List of replication groups that are allowed to create buckets on the corresponding
        namespace
        :param disallowed_vpools_list: List of replication groups that are not allowed to create buckets on the
        corresponding namespace
        :param namespace_admins: Comma separated list of namespace admins
        :param external_group_admins: List of groups from AD Server
        :param user_mapping: List of user mapping objects
        :param is_encryption_enabled: Flag to enable encryption for the namespace
        :param is_stale_allowed: Flag to allow stale data within the namespace
        :param is_compliance_enabled: Flag to enable namespace compliance
        :returns The namespace created
        """
        payload = {
            "namespace": name,
            "default_object_project": default_object_project,
            "default_data_services_vpool": default_data_services_vpool,
            "allowed_vpools_list": allowed_vpools_list,
            "disallowed_vpools_list": disallowed_vpools_list,
            "namespace_admins": namespace_admins,
            "user_mapping": user_mapping,
            "is_encryption_enabled": is_encryption_enabled,
            "default_bucket_block_size": default_bucket_block_size,
            "external_group_admins": external_group_admins,
            "is_stale_allowed": is_stale_allowed,
            "compliance_enabled": is_compliance_enabled
        }
        log.info("Creating namespace '{0}'".format(name))
        return self.conn.post('object/namespaces/namespace', json_payload=payload)

    def update(self, namespace_id, default_data_services_vpool=None, vpools_added_to_allowed_vpools_list=[],
               vpools_added_to_disallowed_vpools_list=[], vpools_removed_from_allowed_vpools_list=[],
               vpools_removed_from_disallowed_vpools_list=[], namespace_admins=None, user_mapping=None,
               default_bucket_block_size=None, external_group_admins=None, is_encryption_enabled=None,
               is_stale_allowed=None):
        """
        Updates namespace details like replication group list, namespace admins and user mappings.
        Replication group can be:
            - Added to allowed or disallowed replication group list
            - Removed from allowed or disallowed replication group list

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param namespace_id: Namespace identifier whose details needs to be updated
        :param default_data_services_vpool: Default replication group identifier when creating buckets
        :param vpools_added_to_allowed_vpools_list: List of replication group identifier which will be added in the
        allowed List for allowing namespace access
        :param vpools_added_to_disallowed_vpools_list: List of replication group identifier which will be added in the
        disallowed list for prohibiting namespace access
        :param vpools_removed_from_allowed_vpools_list: List of replication group identifier which will be removed
        from allowed list
        :param vpools_removed_from_disallowed_vpools_list: List of replication group identifier which will be removed
        from disallowed list for removing their prohibition namespace access
        :param namespace_admins: Comma separated list of namespace admins
        :param user_mapping: List of user mapping objects
        :param default_bucket_block_size: Default bucket quota size
        :param external_group_admins: List of groups from AD Server
        :param is_encryption_enabled: Update encryption for the namespace. If null then encryption will not be updated.
        :param is_stale_allowed: Flag to allow stale data within the namespace. If null then stale allowance will not be
        updated
        """
        payload = {
            "default_data_services_vpool": default_data_services_vpool,
            "vpools_added_to_allowed_vpools_list": vpools_added_to_allowed_vpools_list,
            "vpools_added_to_disallowed_vpools_list": vpools_added_to_disallowed_vpools_list,
            "vpools_removed_from_allowed_vpools_list": vpools_removed_from_allowed_vpools_list,
            "vpools_removed_from_disallowed_vpools_list": vpools_removed_from_disallowed_vpools_list,
            "namespace_admins": namespace_admins,
            "user_mapping": user_mapping,
            "default_bucket_block_size": default_bucket_block_size,
            "external_group_admins": external_group_admins,
            "is_encryption_enabled": is_encryption_enabled,
            "is_stale_allowed": is_stale_allowed
        }
        # FIXME: According to the API, this call should return the updated object, but it does not
        log.info("Updating namespace ID '{}'".format(namespace_id))
        return self.conn.put('object/namespaces/namespace/{}'.format(namespace_id), json_payload=payload)

    def delete(self, namespace_id):
        """
        Deactivates and deletes the given namespace and all associated user mappings.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param namespace_id: An active namespace identifier which needs to be deleted
        """
        log.info("Deleting namespace ID '{}'".format(namespace_id))
        # FIXME: This should be a DELETE request
        return self.conn.post('object/namespaces/namespace/{}/deactivate'.format(namespace_id))

    def get_retention_classes(self, namespace):
        """
        Gets the list of retention classes for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {u'retention_class': []}

        :param namespace: Namespace identifier for which retention classes
        needs to retrieved
        """
        log.info("Getting retention for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/retention'.format(namespace))

    def get_retention_class(self, name, namespace):
        """
        Gets the retention period for the given namespace and retention class.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "name": "class_9cc8777c-bce0-11e4-8580-0050569c6fd7",
            "period": 2
        }

        :param name: Class name for which retention period needs to
        be retrieved
        :param namespace: Namespace for which retention period needs to
        be retrieved
        """
        log.info("Getting retention '{0}' in ns {1}".format(name, namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/retention/{1}'.format(
                namespace, name))

    def create_retention_class(self, name, period, namespace):
        """
        Creates a retention class for the specified namespace. The method
        payload specifies the retention class details which define a name for
        the class and a retention period.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param name: Name of the retention class
        :param period: Period of the retention class in seconds
        :param namespace: Namespace identifier for which retention class needs
        to be created.
        """
        payload = {
            "name": name,
            "period": period
        }

        log.info("Creating retention '{0}' in ns {1}".format(name, namespace))

        return self.conn.post(
            url='object/namespaces/namespace/{0}/retention'.format(namespace),
            json_payload=payload
        )

    def update_retention_class(self, name, period, namespace):
        """
        Updates the retention class details for a specified retention class
        for a namespace.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param name: Retention class name for which details needs to updated.
        :param period: A new period value for class in seconds
        :param namespace: Namespace identifier for which retention class needs
        to be retrieved.
        """
        payload = {
            "period": period
        }

        log.info("Updating retention '{0}' in ns {1}: {2}".format(name,
                                                                  namespace,
                                                                  payload))
        return self.conn.put(
            url='object/namespaces/namespace/{0}/retention/{1}'.format(
                namespace, name), json_payload=payload)

    def get_namespace_quota(self, namespace):
        """
        Gets the namespace quota for a specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blockSize': -1,
            u'notificationSize': -1,
            u'namespace': u'namespace1'
        }

        :param namespace: Namespace identifier for which namespace quota
        details needs to retrieved.
        """
        log.info("Getting quota for namespace '{0}'".format(namespace))

        return self.conn.get(
            url='object/namespaces/namespace/{0}/quota'.format(namespace))

    def update_namespace_quota(self, block_size, notification_size, namespace):
        """
        Updates the namespace quota for a specified namespace.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param block_size: Block size in GB.
        :param notification_size: Notification size in GB.
        :param namespace: Namespace identifier for which namespace quota
        details need to be updated.
        """
        # ----
        # Note
        # ----
        # These two params are what the ECS API docs state, however, every
        # other parameter is broken up into two word with an underscore
        # separating them, ie: block_size
        # I can't verify at the moment that this call works as is.
        payload = {
            "blockSize": block_size,
            "notificationSize": notification_size
        }

        log.info("Updating quota for namespace '{0}': {1}".format(namespace,
                                                                  payload))
        return self.conn.put(
            url='object/namespaces/namespace/{0}/quota'.format(namespace),
            json_payload=payload
        )



# Standard lib imports
import logging
import os

# Third party imports
import requests

# Project level imports
from ecsclient.common.exceptions import ECSClientException


# Suppress the insecure request warning
# https://urllib3.readthedocs.org/en/
# latest/security.html#insecurerequestwarning
requests.packages.urllib3.disable_warnings()

log = logging.getLogger(__name__)


class TokenRequest(object):
    """
    This is a helper class to fetch a new token from the ECS controller
    and return the token as well as store it locally. Prior to fetching a new
    token we check if we have a local token and if so, whether or not it is
    still valid
    """

    def __init__(self, username, password, ecs_endpoint, token_endpoint,
                 verify_ssl, token_path, request_timeout,
                 cache_token):
        """
        Create a new TokenRequest instance

        :param username: The username to fetch a token
        :param password: The password to fetch a token
        :param ecs_endpoint: The URL where ECS is located
        :param token_endpoint: The URL where the ECS login is located
        :param verify_ssl: Verify SSL certificates
        :param token_path: Path to the cached token file
        :param request_timeout: How long to wait for ECS to respond
        :param cache_token: Whether to cache the token, by default this is true
        you should only switch this to false when you want to directly fetch
        a token for a user
        """
        self.username = username
        self.password = password
        self.ecs_endpoint = ecs_endpoint
        self.token_endpoint = token_endpoint
        self.verify_ssl = verify_ssl
        self.token_verification_endpoint = ecs_endpoint + '/user/whoami'
        self.token_path = token_path
        self.request_timeout = request_timeout
        self.cache_token = cache_token
        self.token = None
        self.session = requests.Session()

    def get_new_token(self):
        """
        Request a new authentication token from ECS and persist it
        to a file for future usage if cache_token is true

        :return: Returns a valid token, or None if failed
        """
        log.info("Getting new token")
        self.session.auth = (self.username, self.password)

        req = self.session.get(self.token_endpoint,
                               verify=self.verify_ssl,
                               headers={'Accept': 'application/json'},
                               timeout=self.request_timeout)

        if req.status_code == 401:
            msg = 'Invalid username or password'
            log.fatal(msg)
            raise ECSClientException.from_response(req, message=msg)
        if req.status_code != 200:
            msg = 'Non-200 status returned ({0})'.format(req.status_code)
            log.fatal(msg)
            raise ECSClientException.from_response(req, message=msg)

        self.token = req.headers['x-sds-auth-token']

        if self.cache_token:
            log.debug("Caching token to '{0}'".format(self.token_path))

            token_dir = os.path.dirname(os.path.abspath(self.token_path))
            if not os.path.isdir(token_dir):
                raise ECSClientException('Token directory not found')

            with open(self.token_path, 'w') as token_file:
                token_file.write(self.token)

        return self.token

    def get_token(self):
        """
        Attempt to get an existing token, if successful then ensure it
        hasn't expired yet. If its expired, fetch a new token

        :return: A token
        """
        token = self._get_existing_token()

        if not token:
            log.debug("No Token found getting new one")
            return self.get_new_token()

        # FIXME: Avoid validation at every call
        log.debug("Validating token")
        req = self._request(token, self.token_verification_endpoint)

        if req.status_code == 200:
            log.debug("Token validated successfully")
            return token
        elif req.status_code in (401, 403, 415):
            msg = "Invalid token. Trying to get a new one (Code: {})".format(req.status_code)
            log.warning(msg)
            return self.get_new_token()
        else:  # i.e. 500 or unknown raise an exception
            msg = "Token validation error (Code: {})".format(req.status_code)
            log.error(msg)
            raise ECSClientException.from_response(req, message=msg)

    def _get_existing_token(self):
        """
        Attempt to open and read the token file if it exists

        :return: If available return the token, if not return None
        """

        token = self.token

        if not token and self.cache_token:
            if os.path.isfile(self.token_path):
                log.debug("Reading cached token at '{0}'".format(self.token_path))
                with open(self.token_path, 'r') as token_file:
                    token = token_file.read()

        if not token:
            log.debug("No token found")

        return token

    def _request(self, token, url):
        """
        Perform a request and place the token header inside the request header

        :param token: A valid token
        :param url:  The URL to perform a request
        :return: Request Object
        """
        headers = {'Accept': 'application/json',
                   'X-SDS-AUTH-TOKEN': token}
        try:
            return self.session.get(url, verify=self.verify_ssl,
                                    headers=headers,
                                    timeout=self.request_timeout)
        except requests.ConnectionError as conn_err:
            msg = 'Connection error: {0}'.format(conn_err.args)
            log.error(msg)
            raise ECSClientException(msg)
        except requests.HTTPError as http_err:
            msg = 'HTTP error: {0}'.format(http_err.args)
            log.error(msg)
            raise ECSClientException(msg)
        except requests.RequestException as req_err:
            msg = 'Request error: {0}'.format(req_err.args)
            log.error(msg)
            raise ECSClientException(msg)



import os
import re
import unittest

from six.moves import configparser

from ecsclient.client import Client
from ecsclient.common.util import is_valid_response


class BaseTestCase(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(BaseTestCase, self).__init__(*args, **kwargs)
        self.skip_tests = False
        self._get_config()

    def _get_config(self):
        config_file = os.environ.get('ECS_TEST_CONFIG_FILE',
                                     os.path.join(os.getcwd(), "tests/test.conf"))
        config = configparser.ConfigParser()
        config.read(config_file)
        self.config = config
        if config.has_section('func_test'):
            self.token_endpoint = config.get('func_test', 'token_endpoint')
            self.ecs_endpoint = config.get('func_test', 'ecs_endpoint')
            self.username = config.get('func_test', 'username')
            self.password = config.get('func_test', 'password')
            self.api_version = config.get('func_test', 'api_version')
            license_file = config.get('func_test', 'license_file')
            with open(license_file) as f:
                self.license_text = f.read()
        else:
            self.skip_tests = True

    def _get_client(self):
        return Client(
            self.api_version,
            username=self.username,
            password=self.password,
            ecs_endpoint=self.ecs_endpoint,
            token_endpoint=self.token_endpoint)

    def setUp(self):
        super(BaseTestCase, self).setUp()
        if self.skip_tests:
            self.skipTest('SKIPPING FUNCTIONAL TESTS DUE TO NO CONFIG')
        self.client = self._get_client()

    def tearDown(self):
        super(BaseTestCase, self).tearDown()

    def assertSameCertificate(self, first, second, msg=None):
        """Fail if the two certificates are not equal.
        """
        first = re.sub(r"\s+", "", first)
        second = re.sub(r"\s+", "", second)
        self.assertEqual(first, second, msg=msg)

    def assertValidSchema(self, object, schema):
        """Fail if the object does not comply with the given schema.
        """
        self.assertTrue(is_valid_response(object, schema), 'Response is not valid with the schema')



import logging

log = logging.getLogger(__name__)


class DataStore(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Gets list of configured commodity or filesystem data stores.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'data_store': [
                {
                    u'link': {
                        u'href': u'/vdc/data-stores/commodity/192.29.3.51',
                        u'rel': u'self'
                    },
                    u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
                    u'resource_type': u'commodity',
                    u'id': u'192.29.3.51'
                },
                {
                    u'link': {
                        u'href': u'/vdc/data-stores/commodity/192.29.3.54',
                        u'rel': u'self'
                    },
                    u'name': u'tiva01-r01-07.blah02s2.rraas-ops.com',
                    u'resource_type': u'commodity',
                    u'id': u'192.29.3.54'
                }
            ]
        }
        """
        log.info("Listing data stores")
        return self.conn.get(url='vdc/data-stores')

    def get(self, data_store_id):
        """
        Gets the details for a commodity data store.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
            u'tags': [],
            u'device_state': u'readytouse',
            u'global': None,
            u'creation_time': 1432185210000,
            u'device_info': u'',
            u'used_gb': 1073,
            u'usable_gb': 334800,
            u'vdc': None,
            u'link': {
                u'href': u'/vdc/object-pools/172.29.3.151',
                u'rel': u'self'
            },
            u'free_gb': 333727,
            u'varray': u'urn: storageos: VirtualArray:
                        3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
            u'id': u'192.29.3.51',
            u'description': u''
        }

        :param data_store_id: Identifier of the data store
        """
        log.info("Getting commodity store ID '{}'".format(data_store_id))
        return self.conn.get('vdc/data-stores/commodity/{}'.format(data_store_id))

    def get_by_storage_pool(self, storage_pool_id):
        """
        Gets the list of details of commodity data stores associated with a
        storage pool.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'commodity_data_store': [
                {
                    u'remote': None,
                    u'name': u'tiva01-r01-04.blah01s1.rraas-ops.com',
                    u'tags': [],
                    u'device_state': u'readytouse',
                    u'global': None,
                    u'creation_time': 1432185210000,
                    u'device_info': u'',
                    u'used_gb': 318025,
                    u'usable_gb': 334800,
                    u'vdc': None,
                    u'link': {
                        u'href': u'/vdc/object-pools/192.29.3.51',
                        u'rel': u'self'
                    },
                    u'free_gb': 16775,
                    u'varray': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    u'id': u'192.29.3.51',
                    u'description': u''
                },
                {
                    u'remote': None,
                    u'name': u'tiva01-r01-07.blah02s2.rraas-ops.com',
                    u'tags': [

                    ],
                    u'device_state': u'readytouse',
                    u'global': None,
                    u'creation_time': 1432185214000,
                    u'device_info': u'',
                    u'used_gb': 318026,
                    u'usable_gb': 334800,
                    u'vdc': None,
                    u'link': {
                        u'href': u'/vdc/object-pools/192.29.3.54',
                        u'rel': u'self'
                    },
                    u'free_gb': 16774,
                    u'varray': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37',
                    u'id': u'192.29.3.54',
                    u'description': u''
                }
            ]
        }

        :param storage_pool_id: Identifier of the storage pool
        """
        log.info("Getting commodity stores for storage pool ID '{}'"
                 .format(storage_pool_id))
        return self.conn.get('vdc/data-stores/commodity/search/varray/{0}'.format(storage_pool_id))

    def create(self, name, description, node_id, storage_pool_id):
        """
        NOTE: This is an asychronous operation that returns a task object.

        Creates one or more new commodity data stores. A data store represents
        a commodity node providing backing storage for the object store. On
        creating, the data store is associated with the storage pool specified
        in the request.

        Note that the data store creation is an asynchronous operation so a
        successful invocation of this request does not necessarily mean that
        the creation has completed.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'task': [
                {
                    u'resource': {
                        u'name': u'Sample_Volume',
                        u'id': u'urn: storageos:
                                Volume:5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:',
                        u'link': {
                            u'rel': u'self',
                            u'href': u'/block/volumes/urn: storageos: Volume:
                                      5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:'
                        }
                    },
                    u'state': u'pending',
                    u'start_time': u'1379398608574',
                    u'op_id': u'265cf333-76a1-4129-903e-fac63f9b4adc',
                    u'link': {
                        u'rel': u'self',
                        u'href': u'/block/volumes/urn: storageos: Volume:
                                  5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:
                                  /tasks/265cf333-76a1-4129-903e-fac63f9b4adc'
                    }
                }
            ]
        }


        :param name: User provided name (not verified or unique)
        :param description: User provided description (not verified or unique)
        :param node_id: ID of the commodity node
        :param storage_pool_id: Desired storage pool ID for creating data store
        :returns a task object
        """
        payload = {"nodes": [
            {
                "name": name,
                "description": description,
                "nodeId": node_id,
                "virtual_array": storage_pool_id
            }
        ]}

        log.info("Creating data store '{}': {}".format(name, payload))
        return self.conn.post('vdc/data-stores/commodity', json_payload=payload)

    def delete(self, data_store_id):
        """
        Deactivates the commodity node and data store

        Note: Storage will be deleted if it was allocated with the data store.

        Note: Data store deletion is an asynchronous operation, so a successful invocation
        of this request does not necessarily mean that the deletion has completed.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'task': [
                {
                    u'resource': {
                        u'name': u'Sample_Volume',
                        u'id': u'urn: storageos:
                                Volume:5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:',
                        u'link': {
                            u'rel': u'self',
                            u'href': u'/block/volumes/urn: storageos: Volume:
                                      5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:'
                        }
                    },
                    u'state': u'pending',
                    u'start_time': u'1379398608574',
                    u'op_id': u'265cf333-76a1-4129-903e-fac63f9b4adc',
                    u'link': {
                        u'rel': u'self',
                        u'href': u'/block/volumes/urn: storageos: Volume:
                                  5ba5b8d8-a0ca-4827-84f9-c1fef57733f5:
                                  /tasks/265cf333-76a1-4129-903e-fac63f9b4adc'
                    }
                }
            ]
        }

        :param data_store_id: Identifier of data store to delete
        :return: a task object
        """
        log.info("Deleting data store ID '{}'".format(data_store_id))
        return self.conn.post('vdc/data-stores/{}/deactivate'.format(data_store_id))

    def get_task(self, data_store_id, op_id):
        """
        Get the task with the given ID for a specific data store.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'op_id': u'CREATE',
            u'resource': {
                u'link': {u'href': u'/vdc/object-pools/10.1.83.70', u'rel': u'self'},
                u'id': u'10.1.83.70',
                u'name': u'10.1.83.70'
            },
            u'name': u'CREATE',
            u'global': None,
            u'associated_resources': [],
            u'state': u'pending',
            u'vdc': None,
            u'link': {u'href': u'/vdc/data-stores/10.1.83.70/tasks/CREATE',
                      u'rel': u'self'},
            u'remote': None,
            u'restLink': None
        }

        :param data_store_id: Identifier for the data store to query
        :param op_id: Identifier for the task operation of the data store
        """
        log.info("Getting tasks for data store ID '{}'".format(data_store_id))
        return self.conn.get('vdc/data-stores/{}/tasks/{}'.format(data_store_id, op_id))



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestStoragePool(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestStoragePool, self).__init__(*args, **kwargs)
        self.storage_pool_1 = "functional-tests-storagepool-%s" % int(time.time())
        self.storage_pool_2 = self.storage_pool_1 + '_second'
        self.storage_pool_3 = self.storage_pool_1 + '_third'

    def setUp(self):
        super(TestStoragePool, self).setUp()
        r = self.client.storage_pool.create(self.storage_pool_1)
        self.storage_pool_1_id = r['id']
        self.storage_pool_2_id = 'placeholder'
        r = self.client.storage_pool.create(self.storage_pool_3)
        self.storage_pool_3_id = r['id']

    def tearDown(self):
        super(TestStoragePool, self).tearDown()
        for storage_pool_id in [self.storage_pool_1_id,
                                self.storage_pool_2_id,
                                self.storage_pool_3_id]:
            try:
                self.client.storage_pool.delete(storage_pool_id)
            except ECSClientException:
                pass

    def test_storage_pools_list(self):
        response = self.client.storage_pool.list()
        self.assertValidSchema(response, schemas.STORAGE_POOLS)

    def test_storage_pools_get_one(self):
        response = self.client.storage_pool.get(self.storage_pool_1_id)
        self.assertValidSchema(response, schemas.STORAGE_POOL)

    def test_storage_pools_create(self):
        response = self.client.storage_pool.create(self.storage_pool_2)
        self.assertValidSchema(response, schemas.STORAGE_POOL)
        self.assertEqual(response['name'], self.storage_pool_2)

    def test_storage_pools_update(self):
        new_name = self.storage_pool_1 + '-updated'
        # Get the namespace and verify the value of one of its attributes
        response = self.client.storage_pool.get(self.storage_pool_1_id)
        self.assertNotEqual(response['name'], new_name)
        self.assertFalse(response['isProtected'])
        self.assertFalse(response['isColdStorageEnabled'])

        # Update the attribute and check the response
        response = self.client.storage_pool.update(self.storage_pool_1_id,
                                                   name=new_name,
                                                   is_protected=True)
        self.assertEqual(response['name'], new_name)
        self.assertTrue(response['isProtected'])

    def test_storage_pools_delete(self):
        self.client.storage_pool.delete(self.storage_pool_3_id)
        f = self.client.storage_pool.get
        self.assertRaises(ECSClientException, f, self.storage_pool_3_id)



import unittest
from ecsclient.common.util import get_formatted_time_string


class TestCommonFunctions(unittest.TestCase):

    def setUp(self):
        self.time_bucket_no_minute = '2014-11-18T00'
        self.time_bucket_with_minute = '2014-11-18T00:01'

    def test_should_get_properly_formatted_timestamp_no_minute(self):
            self.assertEqual(self.time_bucket_no_minute,
                             get_formatted_time_string(2014, 11, 18, 0, None))

    def test_should_get_properly_formatted_timestamp_with_minute(self):
            self.assertEqual(self.time_bucket_with_minute,
                             get_formatted_time_string(2014, 11, 18, 0, 1))

    def test_should_throw_value_error(self):
            self.assertRaises(ValueError,
                              get_formatted_time_string, 2014, 11, 18, 'abc')



import logging

log = logging.getLogger(__name__)


class ReplicationGroup(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Lists all configured replication groups.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'data_service_vpool': [
                {
                    u'isAllowAllNamespaces': True,
                    u'remote': None,
                    u'name': u'us1',
                    u'tags': [

                    ],
                    u'global': None,
                    u'creation_time': 1434739694878,
                    u'vdc': None,
                    u'inactive': False,
                    u'varrayMappings': [
                        {
                            u'name': u'urn: storageos: VirtualDataCenterData:
                                    7d7d2ed5-e253-4be9-b8bb-5ff5b2697e6f',
                            u'value': u'urn: storageos: VirtualArray:
                                    527caf55-e989-4485-b92d-ce465d20dd56'
                        }
                    ],
                    u'id': u'urn: storageos: ReplicationGroupInfo:
                                61f20dc2-a862-4935-9110-45030f0fe17c: global',
                    u'description': u''
                },
                {
                    u'isAllowAllNamespaces': True,
                    u'remote': None,
                    u'name': u'us2',
                    u'tags': [

                    ],
                    u'global': None,
                    u'creation_time': 1434739694878,
                    u'vdc': None,
                    u'inactive': False,
                    u'varrayMappings': [
                        {
                            u'name': u'urn: storageos: VirtualDataCenterData:
                                        a9faea85-d377-4a42-b5f1-fa15829f0c33',
                            u'value': u'urn: storageos: VirtualArray:
                                        3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37'
                        }
                    ],
                    u'id': u'urn: storageos: ReplicationGroupInfo:
                            c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
                    u'description': u''
                }
            ]
        }
        """
        log.info("Listing replication groups")
        return self.conn.get('vdc/data-service/vpools')

    def get(self, replication_group_id):
        """
        Gets the details for the specified replication group.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'isAllowAllNamespaces': True,
            u'remote': None,
            u'name': u'us2',
            u'tags': [

            ],
            u'global': None,
            u'creation_time': 1434740060384,
            u'vdc': None,
            u'inactive': False,
            u'varrayMappings': [
                {
                    u'name': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'value': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37'
                }
            ],
            u'id': u'urn: storageos: ReplicationGroupInfo:
                        c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
            u'description': u''
        }

        :param replication_group_id: Replication group identifier for which
        details needs to be retrieved
        """
        log.info("Getting replication group with ID='{}'".format(replication_group_id))
        return self.conn.get('vdc/data-service/vpools/{0}'.format(replication_group_id))

    def update(self, replication_group_id, name=None, description=None,
               enable_rebalancing=None, allow_all_namespaces=None):
        """
        Updates the details for a replication group.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param replication_group_id: Replication group identifier for which details need to be updated
        :param name: New name for the replication group
        :param description: New description for the replication group
        :param enable_rebalancing: Set to True to enabled geo rebalancing. False to disable it.
        :param allow_all_namespaces: Allow all namespaces to update True/False
        """
        # Need to check these mandatory arguments, otherwise the API will complain
        if not name:
            ValueError('name argument must be set')
        if not description:
            ValueError('description argument must be set')
        if not allow_all_namespaces:
            ValueError('allow_all_namespaces argument must be set')

        payload = {
            'name': name,
            'description': description,
            'allowAllNamespaces': allow_all_namespaces
        }

        if enable_rebalancing:
            payload['enable_rebalancing'] = enable_rebalancing

        log.info("Updating replication group with ID='{}'".format(replication_group_id))
        return self.conn.put('vdc/data-service/vpools/{}'.format(replication_group_id), json_payload=payload)

    def create(self, name, zone_mappings, description=None, enable_rebalancing=None, allow_all_namespaces=None,
               is_full_rep=None):
        """
        Creates a replication group that includes the specified storage pools (VDC:storage pool tuple).

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            u'isAllowAllNamespaces': True,
            u'remote': None,
            u'name': u'us2',
            u'global': None,
            u'vdc': None,
            u'varrayMappings': [
                {
                    u'name': u'urn: storageos: VirtualDataCenterData:
                                a9faea85-d377-4a42-b5f1-fa15829f0c33',
                    u'value': u'urn: storageos: VirtualArray:
                                3c4e8cca-2e3d-4f8d-b183-1c69ce2d5b37'
                }
            ],
            u'id': u'urn: storageos: ReplicationGroupInfo:
                        c2b0d3c4-c778-4a24-8da5-6a89784c4eeb: global',
            u'description': u''
        }

        :param name: Unique name identifying this classification of replication group
        :param zone_mappings: Mappings between Virtual Data Centers and Storage Pools. List of tuples with the
        following format: (vdc_id, storage_pool_id)
        :param description: Description of this replication group
        :param enable_rebalancing: Set to True to enabled geo rebalancing. False to disable it.
        :param allow_all_namespaces: Allow all namespaces to update True/False
        :param is_full_rep: Full replication flag
        """
        payload = {
            "name": name,
            "description": description,
            "isAllowAllNamespaces": allow_all_namespaces,
            "enable_rebalancing": enable_rebalancing,
            "isFullRep": is_full_rep,
            "zone_mappings": []
        }

        if not zone_mappings:
            raise ValueError('zone_mappings cannot be empty')

        try:
            for mapping in zone_mappings:
                payload['zone_mappings'].append({
                    "name": mapping[0],
                    "value": mapping[1]
                })
        except:
            raise ValueError('zone_mappings does not have the correct format')

        log.info("Creating replication group with name='{}'".format(name))
        return self.conn.post('vdc/data-service/vpools', json_payload=payload)

    def delete(self, replication_group_id):
        """
        Deletes a specified replication group

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param replication_group_id: Replication group to be deleted
        """
        log.info("Deleting replication group with ID='{}'".format(replication_group_id))
        return self.conn.post('vdc/data-service/vpools/{}/deactivate'.format(replication_group_id))

    def add_storage_pool(self, replication_group_id, vdc_id, storage_pool_id):
        """
        Adds a storage pool (as VDC:storage pool tuple) to the specified replication group.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
          "data_service_vpool_varrays": {
            "mappings": {
              "name": "vdc_id",
              "value": "StoragePool_Id"
            }
          }
        }

        :param replication_group_id: Replication group identifier for which storage pool needs to be added
        :param vdc_id: Virtual data center ID
        :param storage_pool_id: Storage pool ID
        """
        payload = {
            "mappings": [
                {
                    "name": vdc_id,
                    "value": storage_pool_id
                }
            ]
        }
        log.info("Adding the storage pool '{}' to the replication group '{}'".format(
            storage_pool_id, replication_group_id))
        return self.conn.put('vdc/data-service/vpools/{}/addvarrays'.format(replication_group_id),
                             json_payload=payload)

    def remove_storage_pool(self, replication_group_id, vdc_id, storage_pool_id):
        """
        Deletes a storage pool (VDC:storage pool tuple) from a specified replication group.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param replication_group_id: Replication group identifier for which storage pool needs to be removed
        :param vdc_id: Virtual data center ID
        :param storage_pool_id: Storage pool ID
        """
        payload = {
            "mappings": [
                {
                    "name": vdc_id,
                    "value": storage_pool_id
                }
            ]
        }
        log.info("Removing the storage pool '{}' from the replication group '{}'".format(
            storage_pool_id, replication_group_id))
        return self.conn.put('vdc/data-service/vpools/{}/removevarrays'.format(replication_group_id),
                             json_payload=payload)



import time

from ecsclient import schemas
from ecsclient.common.exceptions import ECSClientException
from tests import functional


class TestObjectUser(functional.BaseTestCase):
    def __init__(self, *args, **kwargs):
        super(TestObjectUser, self).__init__(*args, **kwargs)
        self.namespace_1 = "functional-tests-namespace-%s" % int(time.time())
        self.object_user_1 = "functional-tests-objectuser-%s" % int(time.time())
        self.object_user_2 = self.object_user_1 + '_second'

    def setUp(self):
        super(TestObjectUser, self).setUp()
        self.client.namespace.create(self.namespace_1)
        self.client.object_user.create(self.object_user_1, self.namespace_1)

    def tearDown(self):
        super(TestObjectUser, self).tearDown()
        for object_user in [self.object_user_1,
                            self.object_user_2]:
            try:
                self.client.object_user.delete(object_user)
            except ECSClientException:
                pass

        self.client.namespace.delete(self.namespace_1)

    def test_object_user_list(self):
        response = self.client.object_user.list()
        self.assertValidSchema(response, schemas.OBJECT_USERS)

    def test_object_user_list_with_namespace(self):
        response = self.client.object_user.list(namespace=self.namespace_1)
        self.assertValidSchema(response, schemas.OBJECT_USERS)

    def test_object_user_get(self):
        response = self.client.object_user.get(self.object_user_1)
        self.assertValidSchema(response, schemas.OBJECT_USER)

    def test_object_user_get_with_namespace(self):
        response = self.client.object_user.get(self.object_user_1, namespace=self.namespace_1)
        self.assertValidSchema(response, schemas.OBJECT_USER)

    def test_object_user_create(self):
        response = self.client.object_user.create(self.object_user_2, self.namespace_1)
        self.assertValidSchema(response, {"type": "object", "properties": {"link": schemas.LINK}, "required": ["link"]})

    def test_object_user_delete(self):
        self.client.object_user.delete(self.object_user_1, namespace=self.namespace_1)
        f = self.client.object_user.get
        self.assertRaises(ECSClientException, f, self.object_user_1, namespace=self.namespace_1)

    def test_object_user_lock_unlock(self):
        r = self.client.object_user.get_lock(self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(r['isLocked'], False)

        self.client.object_user.lock(self.object_user_1, namespace=self.namespace_1)

        r = self.client.object_user.get_lock(self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(r['isLocked'], True)

        self.client.object_user.unlock(self.object_user_1, namespace=self.namespace_1)

        r = self.client.object_user.get_lock(self.object_user_1, namespace=self.namespace_1)
        self.assertEqual(r['isLocked'], False)



import logging

from ecsclient import baseclient
from ecsclient.v2.cas import cas
from ecsclient.v2.configuration import certificate, configuration_properties, licensing
from ecsclient.v2.configuration import feature
from ecsclient.v2.geo_replication import replication_group, temporary_failed_zone
from ecsclient.v2.provisioning import base_url, bucket, data_store, storage_pool, virtual_data_center, node, \
    vdc_keystore
from ecsclient.v2.metering import billing
from ecsclient.v2.monitoring import capacity, dashboard, events, alerts
from ecsclient.v2.multitenancy import namespace
from ecsclient.v2.user_management import authentication_provider, management_user, object_user, secret_key
from ecsclient.v2.other import user_info


# Initialize logger
log = logging.getLogger(__name__)


class Client(baseclient.Client):
    version = 'v2'

    def __init__(self, *args, **kwargs):
        super(Client, self).__init__(*args, **kwargs)

        # Configuration
        self.certificate = certificate.Certificate(self)
        self.configuration_properties = configuration_properties.ConfigurationProperties(self)
        self.licensing = licensing.Licensing(self)
        self.feature = feature.Feature(self)

        # CAS
        self.cas = cas.Cas(self)

        # File system access
        # TODO: self.nfs = nfs.NFS(self)

        # Metering
        self.billing = billing.Billing(self)

        # Migration
        # TODO: self.transformation = transformation.Transformation(self)

        # Monitoring
        self.capacity = capacity.Capacity(self)
        self.dashboard = dashboard.Dashboard(self)
        self.events = events.Events(self)
        self.alerts = alerts.Alerts(self)

        # Multi-tenancy
        self.namespace = namespace.Namespace(self)

        # Geo-replication
        self.replication_group = replication_group.ReplicationGroup(self)
        self.temp_failed_zone = temporary_failed_zone.TemporaryFailedZone(self)

        # Provisioning
        self.base_url = base_url.BaseUrl(self)
        self.bucket = bucket.Bucket(self)
        self.data_store = data_store.DataStore(self)
        self.node = node.Node(self)
        self.storage_pool = storage_pool.StoragePool(self)
        self.vdc = virtual_data_center.VirtualDataCenter(self)
        self.vdc_keystore = vdc_keystore.VdcKeystore(self)

        # Support
        # TODO: self.call_home = call_home.CallHome(self)

        # User Management
        self.authentication_provider = authentication_provider.AuthenticationProvider(self)
        # TODO: self.password_group = password_group.PasswordGroup(self)
        self.secret_key = secret_key.SecretKey(self)
        self.management_user = management_user.ManagementUser(self)
        self.object_user = object_user.ObjectUser(self)

        # Other
        self.user_info = user_info.UserInfo(self)



import logging

log = logging.getLogger(__name__)


class Feature(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get_sse(self):
        """
        Returns the feed for the details of ServerSideEncryption feature

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:
        {}
        """
        # TODO: Add example JSON response
        log.info("Getting Server Side Encryption details")
        return self.conn.get('feature/ServerSideEncryption')



import requests
import testtools
from requests_mock.contrib import fixture

from ecsclient.common.exceptions import ECSClientException


class TestException(testtools.TestCase):
    TEST_URL = 'https://127.0.0.1:4443/foor/bar?param1=value1&param2=value2'

    def setUp(self):
        super(TestException, self).setUp()
        self.requests_mock = self.useFixture(fixture.Fixture())

    def test_is_exception(self):
        self.assertTrue(issubclass(ECSClientException, Exception))

    def test_attrs(self):
        test_kwargs = {
            'ecs_code': '21000',
            'ecs_retryable': True,
            'ecs_description': 'Error description',
            'ecs_details': 'Error details',
            'http_scheme': 'https',
            'http_host': 'localhost',
            'http_port': 4443,
            'http_path': '/path',
            'http_query': 'param1=value1',
            'http_status': 500,
            'http_reason': 'Reason',
            'http_response_content': 'Response content',
            'http_response_headers': [{'x-header-1': 'value1'}]
        }
        exc = ECSClientException('test', **test_kwargs)

        for key, value in test_kwargs.items():
            self.assertIs(True, hasattr(exc, key))
            self.assertEqual(getattr(exc, key), value)

    def test_format(self):
        test_kwargs = {
            'ecs_description': 'Error description',
            'ecs_details': 'Error details',
            'http_scheme': 'https',
            'http_host': 'localhost',
            'http_port': 4443,
            'http_path': '/path',
            'http_query': 'param1=value1',
            'http_status': 500,
            'http_reason': 'Reason',
            'http_response_content': 'Response content'
        }

        for key, value in test_kwargs.items():
            kwargs = {key: value}
            exc = ECSClientException('test', **kwargs)
            self.assertIn(str(value), str(exc))

    def test_from_response(self):
        response_content = '{"code": "1000", "retryable": false, ' \
                           '"description": "Error description", ' \
                           '"details": "Error details"}'
        self.requests_mock.register_uri('POST', self.TEST_URL,
                                        status_code=500,
                                        text=response_content)
        resp = requests.post(self.TEST_URL, data='body')
        exc = ECSClientException.from_response(resp)

        self.assertEqual(exc.ecs_code, '1000')
        self.assertEqual(exc.ecs_retryable, False)
        self.assertEqual(exc.ecs_description, 'Error description')
        self.assertEqual(exc.ecs_details, 'Error details')
        self.assertEqual(exc.http_scheme, 'https')
        self.assertEqual(exc.http_host, '127.0.0.1')
        self.assertEqual(exc.http_port, 4443)
        self.assertEqual(exc.http_path, '/foor/bar')
        self.assertEqual(exc.http_query, 'param1=value1&param2=value2')
        self.assertEqual(exc.http_status, 500)
        self.assertEqual(exc.http_reason, None)
        self.assertEqual(exc.http_response_content, response_content)



import logging

log = logging.getLogger(__name__)


class VdcKeystore(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def get(self):
        """
        Get the certificate chain being used by ECS.

        Required role(s):

        This call has no restrictions.

        Example JSON result from the API:

        {
            "chain": "-----BEGIN CERTIFICATE-----\nMIIDBjCCAe4..."
        }
        """
        log.info("Getting the certificate chain")
        return self.conn.get('vdc/keystore')

    def set(self, private_key, certificate_chain):
        """
        Set the private key and certificate chain being used by ECS.
        WARNING: Note that the certificate will not be updated until ECS is
        restarted. Restarting ECS is out of the scope of this library.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        {
            "chain": "-----BEGIN CERTIFICATE-----\nMIIDBjCCAe4..."
        }

        :param private_key: The private key to be set
        :param certificate_chain: The certificate chain to be set
        """
        payload = {
          "key_and_certificate": {
            "private_key": private_key,
            "certificate_chain": certificate_chain
          }
        }

        log.info("Setting the private key and certificate chain (ECS must be "
                 "restarted for this change to take effect)")
        return self.conn.put('vdc/keystore', json_payload=payload)



import logging

log = logging.getLogger(__name__)


class ManagementUser(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Gets all configured local management users.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'mgmt_user_info': [
                {
                    u'isSystemMonitor': False,
                    u'isSecurityAdmin': False,
                    u'userId': u'someone@internal',
                    u'isSystemAdmin': True
                },
                {
                    u'isSystemMonitor': False,
                    u'isSecurityAdmin': False,
                    u'userId': u'root',
                    u'isSystemAdmin': True
                }
            ]
        }
        """
        log.info('Listing all local management users')
        return self.conn.get(url='vdc/users')

    def get(self, user_id):
        """
        Gets details for the specified local management user.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'isSystemMonitor': False,
            u'userId': u'admin',
            u'isSystemAdmin': True
            u'isSecurityAdmin': True
        }

        :param user_id: User identifier for which local user information needs to
        be retrieved
        """
        log.info("Getting local management user '{}'".format(user_id))
        return self.conn.get(url='vdc/users/{}'.format(user_id))

    def delete(self, user_id):
        """
        Deletes local management user information for the specified user
        identifier.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: User identifier for which local user information needs to
        be deleted.
        """
        log.info("Deleting local management user '{}'".format(user_id))
        return self.conn.post(url='vdc/users/{}/deactivate'.format(user_id))

    def create(self, user_id, password, is_system_admin=False,
               is_system_monitor=False, is_security_admin=False):
        """
        Creates local users for the VDC. These users can be assigned to
        VDC-wide management roles and are not associated with a namespace.
        User account can be assigned to the System Admin role by setting the
        isSystemAdmin flag in the request payload.

        Required role(s):

        SYSTEM_ADMIN

        :param user_id: Management user id, ex: sampleuser
        :param password: Password for the management user
        :param is_system_admin: If set to True, assigns the management user to
        the System Admin role. Default: False
        :param is_system_monitor: If set to True, assigns the management user
        to the System Monitor role. Default: False
        :param is_security_admin: If set to True, assigns the management user
        to the Security Admin role. Default: False
        """
        payload = {
            "userId": user_id,
            "password": password,
            "isSystemAdmin": is_system_admin,
            "isSystemMonitor": is_system_monitor,
            "isSecurityAdmin": is_security_admin
        }

        log.info("Creating local management user '{}'".format(user_id))
        return self.conn.post(url='vdc/users', json_payload=payload)

    def update(self, user_id, password, is_system_admin=False,
               is_system_monitor=False, is_security_admin=False):
        """
        Updates user details for the specified local management user.

        Required role(s):

        SYSTEM_ADMIN

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param user_id: User identifier for which local user information needs to
        be updated.
        :param password: Password for the management user
        :param is_system_admin: If set to True, assigns the management user to
        the System Admin role. Default: False
        :param is_system_monitor: If set to True, assigns the management user
        to the System Monitor role. Default: False
        :param is_security_admin: If set to True, assigns the management user
        to the Security Admin role. Default: False
        """
        payload = {
            "password": password,
            "isSystemAdmin": is_system_admin,
            "isSystemMonitor": is_system_monitor,
            "isSecurityAdmin": is_security_admin
        }

        log.info("Updating local management user '{}'".format(user_id))

        return self.conn.put(
            url='vdc/users/{}'.format(user_id), json_payload=payload)



from ecsclient.common.configuration import certificate, configuration_properties, licensing, feature

certificate = certificate
configuration_properties = configuration_properties
licensing = licensing
feature = feature



from ecsclient.common.provisioning import base_url, bucket, data_store, storage_pool, virtual_data_center, vdc_keystore

base_url = base_url
bucket = bucket
data_store = data_store
storage_pool = storage_pool
virtual_data_center = virtual_data_center
vdc_keystore = vdc_keystore



import json
import logging

log = logging.getLogger(__name__)


class Bucket(object):
    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def create(self, bucket_name, replication_group='', filesystem_enabled=False,
               head_type=None, namespace=None, stale_allowed=False,
               metadata=None, encryption_enabled=False):
        """
        Creates a bucket which could be used by users to create objects.
        The bucket is created in a storage pool associated with the specified
        replication group.

        - Current user will become the bucket owner.
        - If namespace to this bucket creation does not exist, user's namespace
          is used
        - For non SYSTEM_ADMIN user, Namespace should be current user's
          namespace
        - Optional searchable metadata should be an array of JSON dictionaries,
          e.g. [{"name" : "x-amz-meta-custom", "type" : "User", "datatype" : "string"}, ...]


        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            "name": "bucket5",
            "id": "s3.bucket5",
            "inactive": false,
            "global": null,
            "remote": null,
            "vdc": null,
            "tags": [],
            "search_metadata":{
              "metadata":[
                 {
                    "type": "User",
                    "datatype":"integer",
                    "name":"x-amz-meta-custom"
                 }
              ]
            }
        }

        :param bucket_name: The bucket name
        :param replication_group: Replication group identifier
        :param filesystem_enabled: Boolean indicating whether file-system is enabled for this bucket
        :param head_type: Indicates the object head type that is allowed to access the bucket.
        If the bucket has FS-Enabled, then the FS heads are implicitly allowed to access this bucket
        :param namespace: Namespace associated with the user/tenant that is allowed to access the bucket
        :param stale_allowed: Boolean indicating whether to allow stale data in bucket
        :param metadata: Searchable tags assigned to objects created within the bucket.
        Example: [{"name" : "x-amz-meta-custom", "type" : "User", "datatype" : "string"}, ...]
        :param encryption_enabled: Boolean indicating whether is enabled for the bucket
        """
        payload = {
            "name": bucket_name,
            "vpool": replication_group,
            "filesystem_enabled": filesystem_enabled,
            "is_stale_allowed": stale_allowed,
            "is_encryption_enabled": encryption_enabled,
        }

        log.info("Creating bucket '{}'".format(bucket_name))

        if namespace:
            payload['namespace'] = namespace
        if head_type:
            payload['head_type'] = head_type
        if metadata:
            payload['search_metadata'] = metadata

        return self.conn.post('object/bucket', json_payload=payload)

    def delete(self, bucket_name, namespace=None):
        """
        Deletes the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: The bucket name to be deleted
        :param namespace: Namespace associated. If not provided, then current user's namespace is used
        """
        msg = "Deleting bucket '{}'".format(bucket_name)
        url = 'object/bucket/{}/deactivate'.format(bucket_name)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.post(url)

    def list(self, namespace, marker='', limit=100):
        """
        Gets the list of buckets for the specified namespace.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'MaxBuckets': 100,
            u'NextMarker': u'blah-06860733-a0a8-45b5-b4ac-b18ee1e4257d',
            u'object_bucket': [
                {
                    u'remote': None,
                    u'name': u'my-test',
                    u'tags': [
                    ],
                    u'fs_access_enabled': False,
                    u'global': None,
                    u'namespace': u'namespace1',
                    u'api_type': u'S3',
                    u'created': u'2015-06-15T18: 39: 11.957Z',
                    u'vdc': None,
                    u'default_retention': -2,
                    u'is_stale_allowed': False,
                    u'owner': u'user1',
                    u'locked': False,
                    u'notification_size': -1,
                    u'block_size': -1,
                    u'vpool': u'urn: storageos: ReplicationGroupInfo:
                                3ba1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global'
                },
                {
                    u'remote': None,
                    u'name': u'large-uploads',
                    u'tags': [
                    ],
                    u'fs_access_enabled': False,
                    u'global': None,
                    u'namespace': u'namespace1',
                    u'api_type': u'S3',
                    u'created': u'2015-06-12T17: 35: 53.366Z',
                    u'vdc': None,
                    u'default_retention': -2,
                    u'is_stale_allowed': False,
                    u'owner': u'fancyuser1',
                    u'locked': False,
                    u'notification_size': -1,
                    u'block_size': -1,
                    u'vpool': u'urn: storageos: ReplicationGroupInfo:
                                3ba1fa1e-a7d1-4a8e-b8cc-e5a2c27f308d: global'
                }
            ]
        }

        :param namespace: The namespace to query for buckets
        :param marker: Reference to last object returned
        :param limit: Number of objects requested in current fetch
        """
        log.info("Getting all buckets in namespace '{}'".format(namespace))

        return self.conn.get('object/bucket?namespace={namespace}&marker={marker}&limit={limit}'.format(
            namespace=namespace,
            marker=marker,
            limit=limit))

    def set_retention(self, bucket_name, namespace, period=2592000):
        """
        Updates the default retention setting for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: The bucket name
        :param namespace: The namespace
        :param period: Default retention period for bucket in seconds. Defaults
        to 30 days (2592000 seconds)
        """
        payload = {
            "period": period,
            "namespace": namespace
        }

        log.info("Setting retention for bucket '{0}' in namespace "
                 "'{1}': {2}".format(bucket_name, namespace, payload))

        return self.conn.put('object/bucket/{}/retention'.format(bucket_name),
                             json_payload=payload)

    def get_retention(self, bucket_name, namespace=None):
        """
        Gets the retention setting for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {u'period': 2592000}

        :param bucket_name: Bucket name for which the retention setting will be
        retrieved
        :param namespace: Namespace associated. If it is null, then current
        user's Namespace is used.
        """
        url = 'object/bucket/{}/retention'.format(bucket_name)
        msg = "Getting retention for bucket '{}'".format(bucket_name)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url)

    def get(self, bucket_name, namespace=None):
        """
        Gets bucket information for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'remote': None,
            u'name': u'bucket-test1',
            u'tags': [
            ],
            u'fs_access_enabled': False,
            u'global': None,
            u'namespace': u'namespace1',
            u'api_type': u'S3',
            u'created': u'2015-06-15T21: 26: 52.781Z',
            u'vdc': None,
            u'default_retention': 2592000,
            u'is_stale_allowed': False,
            u'owner': u'admin@internal',
            u'locked': False,
            u'notification_size': -1,
            u'block_size': -1,
            u'vpool': u'urn: storageos: ReplicationGroupInfo:
                        3af1fa7e-a7d1-4a9e-b8cc-e5a2c27f308d: global'
        }

        :param bucket_name: Bucket name for which information will be retrieved
        :param namespace: Namespace associated. If it is null, then current
        user's Namespace is used.
        """
        msg = "Getting info for bucket '{}'".format(bucket_name)
        url = 'object/bucket/{}/info'.format(bucket_name)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url)

    def set_owner(self, bucket_name, new_owner, namespace=None):
        """
        Updates the owner for the specified bucket.

        Required role(s):

        SYSTEM_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param namespace: The namespace
        :param new_owner: New owner must be a valid user
        :param bucket_name: Name of the bucket for which owner will be updated
        """
        payload = {
            "new_owner": new_owner
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating owner for bucket '{}': {}".format(bucket_name, new_owner))
        return self.conn.post('object/bucket/{}/owner'.format(bucket_name),
                              json_payload=payload)

    def set_stale_allowed(self, bucket_name, stale_allowed, namespace=None):
        """
        Updates isStaleAllowed details for the specified bucket. If namespace
        does not exist in the request payload, the current user's namespace
        is used.

        If you set this flag ON, and a temporary site outage occurs, objects that
        you access in this bucket might have been updated at the failed site but
        changes might not have been propagated to the site from which you are
        accessing the object.Hence, you are prepared to accept that the objects
        you read might not be up to date.

        If the flag is turned OFF, data in the zone which has the temporary outage is not
        available for access from other zones and object reads for data which has its
        primary in the failed site will fail.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket for which isStaleAllowed is to
        be updated
        :param stale_allowed: true/false
        :param namespace: Namespace associated with the user/tenant that is
        allowed to access the bucket
        """
        payload = {
            "is_stale_allowed": stale_allowed
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating 'isStaleAllowed' for bucket '{}': {}".format(
            bucket_name, stale_allowed))

        return self.conn.post('object/bucket/{}/isstaleallowed'.format(bucket_name),
                              json_payload=payload)

    def get_lock(self, bucket_name, namespace=None):
        """
        Gets lock information for the specified bucket. The current user's
        namespace is used.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'isLocked': False,
            u'bucket_name': u'bucket-test1'
        }

        :param bucket_name: Example: The bucket name to get lock information
        :param namespace: Name of the bucket for which lock information is to
        be retrieved
        """
        msg = "Getting lock info for bucket '{}'".format(bucket_name)
        url = 'object/bucket/{}/lock'.format(bucket_name)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url)

    def set_lock(self, bucket_name, is_locked=False, namespace=None):
        """
        Locks or unlocks the specified bucket. Current user's namespace
        is used.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket which is to be locked/unlocked.
        :param is_locked: Set to True to lock the bucket and False to unlock the
        bucket
        :param namespace: The namespace
        """
        payload = {}
        if namespace:
            payload['namespace'] = namespace

        log.info("Setting lock to '{}' for bucket '{}'".format(
            is_locked, bucket_name))

        return self.conn.put(
            'object/bucket/{}/lock/{}'.format(bucket_name, json.dumps(is_locked)),
            json_payload=payload)

    def set_quota(self, bucket_name, block_size, notification_size, namespace=None):
        """
        Updates the quota for the specified bucket. The payload specifies a limit
        at which a notification will be raised in the event log and a limit at
        which access will be blocked.

        Both notification and block values must be supplied. If you do not want
        to define one of them, you can set its value to -1. You cannot set both
        values to -1 using this API. To set both notification and block values
        to -1, use the delete quota API.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket for which the quota is to be
        updated
        :param block_size: Block size in GB. Cannot be less than 1GB or use
        decimal values. Can be set to -1 to indicate quota value not defined
        :param notification_size: Notification size in GB. Cannot be less than
        1GB or use decimal values. Can be set to -1 to indicate quota value not
        defined
        :param namespace: Namespace to which this bucket belongs
        """
        payload = {
            "blockSize": block_size,
            "notificationSize": notification_size
        }

        if namespace:
            payload['namespace'] = namespace

        log.info("Updating quota for bucket '{}'".format(bucket_name))
        return self.conn.put(
            'object/bucket/{0}/quota'.format(bucket_name),
            json_payload=payload)

    def get_quota(self, bucket_name, namespace=None):
        """
        Gets the quota for the given bucket and namespace. The namespace with
        which the bucket is associated can be specified as a query parameter.

        A value of -1 for the block or notification quota value indicates that
        no quota has been defined.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'blockSize': 1,
            u'notificationSize': 2,
            u'namespace': u'namespace1',
            u'bucketname': u'bucket-test1'
        }

        :param bucket_name: Name of the bucket which for which quota is to be
        retrieved
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        msg = "Getting quota for bucket '{}'".format(bucket_name)
        url = 'object/bucket/{}/quota'.format(bucket_name)

        if namespace:
            url += '?namespace={}'.format(namespace)
            msg += " in namespace '{}'".format(namespace)

        log.info(msg)
        return self.conn.get(url)

    def delete_quota(self, bucket_name, namespace=None):
        """
        Deletes the quota setting for the given bucket and namespace.
        The namespace with which the bucket is associated can be specified as
        a query parameter.

        Required role(s):

        SYSTEM_ADMIN
        NAMESPACE_ADMIN

        Example JSON result from the API:

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Name of the bucket for which the quota is to
        be deleted
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        params = None

        if namespace:
            params = {'namespace': namespace}

        log.info("Deleting quota for bucket '{}'".format(bucket_name))

        return self.conn.delete(
            url='object/bucket/{}/quota'.format(bucket_name),
            params=params
        )

    def get_acl(self, bucket_name, namespace=None):
        """
        Gets the ACL for the given bucket. Current user's namespace is used.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR

        Example JSON result from the API:

        {
            u'namespace': u'namespace1',
            u'bucket': u'bucket-test1',
            u'permission': []
        }

        :param bucket_name: Name of the bucket.
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used.
        """
        params = None
        if namespace:
            params = {'namespace': namespace}

        log.info("Getting ACL for bucket '{}'".format(bucket_name))

        return self.conn.get(
            'object/bucket/{}/acl'.format(bucket_name),
            params=params)

    def set_acl(self, bucket_name, namespace=None, owner=None, default_group=None,
                user_acl=None, group_acl=None, customgroup_acl=None):
        """
        Sets the ACL for the given bucket. If the buckets's namespace is not
        specified in the payload, the current user's namespace is used.

        Required role(s):

        This call has no restrictions

        :param bucket_name: The name of bucket used to set ACL information
        :param namespace: The namespace to which the bucket belongs. If not provided,
        then current user's namespace is used
        :param owner: The name of bucket owner
        :param default_group: The default group of the bucket
        :param user_acl: A collection of users and their corresponding permissions
        (e.g. `[{'permission': ['full_control'], 'user': 'myuser1'}]`)
        :param group_acl: A collection of groups and their corresponding permissions
        (e.g. `[{'permission': ['read'], 'group': 'public'}]`)
        :param customgroup_acl: A collection of custom groups and their corresponding permissions
        (e.g. `[{'permission': ['delete', 'read', 'write'], 'customgroup': 'cgroup1'}]`)
        """
        payload = {
            "bucket": bucket_name,
            "acl": {}
        }

        if namespace:
            payload['namespace'] = namespace
        if owner:
            payload['acl']['owner'] = owner
        if default_group:
            payload['acl']['default_group'] = default_group
        if user_acl:
            payload['acl']['user_acl'] = user_acl
        if group_acl:
            payload['acl']['group_acl'] = group_acl
        if customgroup_acl:
            payload['acl']['customgroup_acl'] = customgroup_acl

        log.info("Setting ACL for bucket '{}'".format(bucket_name))
        self.conn.put(
            'object/bucket/{}/acl'.format(bucket_name),
            json_payload=payload)

    def get_acl_permissions(self):
        """
        Gets all ACL permissions.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'permission': [
                {
                    u'display_name': u'read',
                    u'id': u'read'
                },
                {
                    u'display_name': u'readACL',
                    u'id': u'read_acl'
                },
                {
                    u'display_name': u'write',
                    u'id': u'write'
                },
                {
                    u'display_name': u'writeACL',
                    u'id': u'write_acl'
                },
                {
                    u'display_name': u'execute',
                    u'id': u'execute'
                },
                {
                    u'display_name': u'fullcontrol',
                    u'id': u'full_control'
                },
                {
                    u'display_name': u'privilegedwrite',
                    u'id': u'privileged_write'
                },
                {
                    u'display_name': u'delete',
                    u'id': u'delete'
                },
                {
                    u'display_name': u'none',
                    u'id': u'none'
                }
            ]
        }
        """
        log.info('Getting all ACLs')
        return self.conn.get('object/bucket/acl/permissions')

    def get_acl_groups(self):
        """
        Gets all ACL groups.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'group': [
                {
                    u'display_name': u'public',
                    u'id': u'public',
                    u'description': u'allusersincludingauthenticatedusers&anon'
                },
                {
                    u'display_name': u'allusers',
                    u'id': u'all_users',
                    u'description': u'allauthenticatedusers'
                },
                {
                    u'display_name': u'logdelivery',
                    u'id': u'log_delivery',
                    u'description': u'specifictoS3'
                },
                {
                    u'display_name': u'other',
                    u'id': u'other',
                    u'description': u'allauthenticatedusersbuttheowner'
                }
            ]
        }
        """
        log.info('Getting all ACL groups')
        return self.conn.get('object/bucket/acl/groups')

    def get_metadata(self, bucket_name, head_type, namespace=None):
        """
        Fetch a page of head-specific metadata for the specified bucket

        Required Role(s):

        This call has no restrictions.

        Example JSON result from the API:

        {
            'metadata': [
                {'name': 'key1', 'value': 'value1'},
                {'name': 'key2', 'value': 'value2'}
            ],
            'head_type': 'S3'
        }

        :param bucket_name: Name of the bucket for which the default group
        is to be updated
        :param head_type: the head-type of the metadata to be queried (HDFS,
        S3, etc)
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used
        """
        params = {}
        if namespace:
            params['namespace'] = namespace
        if head_type:
            params['headType'] = head_type

        log.info("Getting metadata for bucket '{}'".format(bucket_name))
        return self.conn.get(
            'object/bucket/{}/metadata'.format(bucket_name),
            params=params)

    def set_metadata(self, bucket_name, metadata_key, metadata_value, head_type,
                     namespace=None):
        """
        Persist additional head metadata for the bucket

        Required Role(s):

        This call has no restrictions.

        Example JSON result from the API:

        :param bucket_name: Name of the bucket for which the metadata is to be added
        :param metadata_key: Metadata key to be added
        :param metadata_value: Metadata value to be added
        :param head_type: the head-type of the metadata to be added (HDFS,
        S3, etc)
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used
        """
        payload = {
            "head_type": head_type,
            "metadata": [
                {
                    "name": metadata_key,
                    "value": metadata_value
                }
            ]
        }
        url = 'object/bucket/{}/metadata'.format(bucket_name)
        if namespace:
            url += '?namespace={}'.format(namespace)
            payload['namespace'] = namespace

        log.info("Adding metadata to bucket '{}'".format(bucket_name))
        return self.conn.put(url, json_payload=payload)

    def delete_metadata(self, bucket_name, head_type, namespace=None):
        """
        Delete a page of head metadata for the specified bucket

        Required Role(s):

        This call has no restrictions.

        Example JSON result from the API:

        :param bucket_name: name of the bucket for which the metadata is to be removed
        :param head_type: the head-type of the metadata to be removed (HDFS, S3, etc)
        :param namespace: Namespace with which bucket is associated. If it is
        null, the current user's namespace is used
        """
        params = {'headType': head_type}
        if namespace:
            params['namespace'] = namespace

        log.info("Deleting metadata for bucket '{}', head '{}'".format(bucket_name, head_type))
        return self.conn.delete(
            'object/bucket/{}/metadata'.format(bucket_name),
            params=params)

    def get_system_metadata_keys(self):
        """
        Lists the system metadata keys available.

        Required Role(s):

        This call has no restrictions.

        Example JSON result from the API:

        ...
        """
        log.info('Listing the system metadata keys available')
        return self.conn.get('object/bucket/searchmetadata')

    def disable_search_metadata(self, bucket_name, namespace=None):
        """
        Disables the metadata search functionality for a bucket.

        Required Role(s):

        This call has no restrictions.

        There is no response body for this call

        Expect: HTTP/1.1 200 OK

        :param bucket_name: Bucket name for which metadata search mode will
        be disabled
        :param namespace: Namespace associated. If it is null, then current
        user's namespace is used
        """
        params = None
        if namespace:
            params = {'namespace': namespace}

        log.info("Disabling search metadata functionality for bucket '{}'".format(bucket_name))

        return self.conn.delete(
            'object/bucket/{}/searchmetadata'.format(bucket_name),
            params=params)



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class Node(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def list(self):
        """
        Gets a list of the data nodes that are currently configured in the cluster.

        Required role(s):

        SYSTEM_ADMIN
        SYSTEM_MONITOR
        NAMESPACE_ADMIN

        Example JSON result from the API:

        {
            u'node': [
                {
                    u'ip': u'172.29.3.148',
                    u'version': u'1.2.0.0.60071.ffbe16c',
                    u'rackId': u'gray',
                    u'nodename': u'supr01-r01-01.lax01s1.rspaas-lab.ops.com',
                    u'nodeid': u'171.29.3.140'
                },
                {
                    u'ip': u'172.29.3.149',
                    u'version': u'1.2.0.0.60071.ffbe16c',
                    u'rackId': u'gray',
                    u'nodename': u'supr01-r01-02.lax01s1.rspaas-lab.ops.com',
                    u'nodeid': u'171.29.3.141'
                }
            ]
        }
        """
        log.info('Getting all nodes for cluster')
        return self.conn.get(url='vdc/nodes')



# Standard lib imports
import logging

# Third party imports
# None

# Project level imports
# None


log = logging.getLogger(__name__)


class UserInfo(object):

    def __init__(self, connection):
        """
        Initialize a new instance
        """
        self.conn = connection

    def whoami(self):
        """
        Example JSON result from the API:

        {
            u'common_name': u'ecsadmin@internal',
            u'distinguished_name': u'',
            u'namespace': u'',
            u'roles': [
                u'SYSTEM_ADMIN'
            ]
        }
        """
        log.info('Getting my own user info (whoami)')
        return self.conn.get('user/whoami')
